#!/bin/sh
#
# BEGIN COPYRIGHT
# *************************************************************************
# Licensed Materials - Property of IBM 
# 5724-L01, 5655-N53, 5724-I82, 5655-R15
# (C) Copyright IBM Corporation 2012. All rights reserved. 
# US Government Users Restricted Rights - Use, duplication, or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
# *************************************************************************
# END COPYRIGHT
# Usage: 
#   createDB2.sh
#     -DBAlias <$DBALIAS>           DB2 CLP alias name (No default)
#     -DBCreate                           Create database (Default false)
#     -DBBP4K <$DBBP4K>        4K buffer pool value (Default BP1)
#     -DBBPIndex <$DBBPINDEX>  Index buffer pool value (Default BP2)
#     -DBName <$DBNAME>             Database name (Default BPMDB)
#     -DBSto <$DBSTOGRP>      StorageGroup Name (Default BPMDBSTO)
#     -DBUser <$DBUSER>    DB Userid (Default wsadmin)
#     -DBVolumes <$DBVOLUMES> DASD Volumes (Default '*')
#     -DBVCat <$DBVCAT>    Volume Catalog (Default BPMCAT)
#     -RunSQL     Execute SQL statements (Default true)
# Example:
#   ./createDB2.sh -DBAlias DSNV10PP -DBName S5S1DB -DBSto S5DBSTO -DBVCat DSNV10PP -DBCreate -RunSQL
#------------------------------------------------------------------------------
#
#-----------------------------------------
# Ask user for the Alias for database
#-----------------------------------------
get_aliasname() {
   while true; do
      if [ -n "$DBALIAS" ]; then
         DBALIAS=$DBALIAS
         break
      else
         echo         
		 echo "Enter DB2 Location/Alias Name ( required ): "
         read DBALIAS
         continue
      fi
   done
}

 DBALIAS="";

 DBNAME="BPMDB";
 DBCREATE="false";
 DBBP4K="BP1";
 DBBPINDEX="BP2";
 DBUSER="wsadmin";
 DBSTOGRP="BPMDBSTO";
 DBVOLUMES="'*'";
 DBVCAT="BPMCAT";
 DBRunSQL="false";

################################
#  DB_OPTIONS will be replaced 
################################
# Exported as to not overload Command Arguments
OUTFILE=z_output.txt
DB_OPTIONS="UPDATE COMMAND OPTIONS USING z ON $OUTFILE o OFF"

if [ -f $OUTFILE ]; then
   rm $OUTFILE
fi

Skip=0
InvalidOption=0;

if [ $# -lt 2 ] ; then
   echo "\nMissing arguments, exiting script"
   echo "        -DBAlias    DB2 Alias name                   <$DBALIAS>  ";
   echo "        -DBCreate   Create Database                  <$DBCREATE> ";
   echo "        -DBBP4K     Buffer pool value                <$DBBP4K>   ";
   echo "        -DBBPIndex  Buffer pool value                <$DBBPINDEX>";
   echo "        -DBName     DataBase name                    <$DBNAME>   ";
   echo "        -DBSto      StorageGroup Name                <$DBSTOGRP> ";
   echo "        -DBUser     DB Userid                        <$DBUSER>   ";
   echo "        -DBVolumes  DASD Volumes                     <$DBVOLUMES>";
   echo "        -DBVCat     Volume Catalog                   <$DBVCAT>   ";
   echo "        -RunSQL     Execute SQL statements           <$DBRunSQL> ";
   echo "\n\n";
   exit;
fi

for i in $*
   do
      case $i in
      -DBCreate)        shift; DBCREATE=true;;
      -RunSQL)          shift; DBRunSQL=true;;
      -DBAlias)         DBALIAS=$2;Skip=1; shift 2;;
      -DBName)          DBNAME=$2; Skip=1; shift 2;;
      -DBSto)           DBSTOGRP=$2; Skip=1; shift 2;;
      -DBBP4K)          DBBP4K=$2; Skip=1; shift 2;;
      -DBBPIndex)       DBBPINDEX=$2; Skip=1; shift 2;;
      -DBUser)          DBUSER=$2; Skip=1; shift 2;;
      -DBVolumes)       DBVOLUMES=$2; Skip=1; shift 2;;
      -DBVCat)          DBVCAT=$2; Skip=1; shift 2;;
      -DBVCat)          DBVCAT=$2; Skip=1; shift 2;;
      --)               shift; break;;
      *)                # If invalid values encountered, exit script
                        if [ "$Skip" = "1" ]
                           then
                              Skip=0
                        else
                           echo "Invalid Option $i" ;
                           InvalidOption=1;
                        fi;;
      esac
   done

if [ $InvalidOption -eq "1" ] ; then
   echo "Invalid arguments, exiting script"
   exit;
fi

get_aliasname    # prompt for Alias name if none specified

echo  "Creating database schema definitions"

INIT_DIR=`pwd`

CURRENT_DIR=`dirname $0`
if [ "$CURRENT_DIR" = "." ] ; then
   CURRENT_DIR=`pwd`
fi

#Specify Connect to the database specifying the id and the password or Alias   TO <xxx> USER <xxx> USING <xxxx>
rc=0

echo '\nCONNECT TO '$DBALIAS';\n'$DB_OPTIONS';\n' > z_schema.ddl

if [ "$DBCREATE" = "true" ]; then
   echo 's/#STOGRP#/'$DBSTOGRP'/g'        > cdb.sed
   echo 's/#VOLUMES#/'$DBVOLUMES'/g'     >> cdb.sed
   echo 's/#VCAT#/'$DBVCAT'/g'           >> cdb.sed
   echo 's/#BPTABLE4K#/'$DBBP4K'/g'      >> cdb.sed
   echo 's/#BPINDEX#/'$DBBPINDEX'/g'     >> cdb.sed
   echo 's/#DB_USER#/'$DBUSER'/g'        >> cdb.sed
   echo 's/#DB_NAME#/'$DBNAME'/g'        >> cdb.sed
   iconv -t IBM-1047 -f ISO8859-1 dropDatabase.sql   | sed -f cdb.sed  >> z_schema.ddl
   iconv -t IBM-1047 -f ISO8859-1 createDatabase.sql | sed -f cdb.sed  >> z_schema.ddl
   rm cdb.sed
fi

iconv -f IBM-1047 -t ISO8859-1 z_schema.ddl > z_schema.sql
rm z_schema.ddl

if [ $rc -eq 0 ] ; then
    for i  in `ls *.sql | egrep "(^create)Tablespace"`
	do
        cat $i >> z_schema.sql    #cat files to reduce connects
	done

    for i  in `ls *.sql | egrep "(^create|^insert)Schema"`
	do
        cat $i >> z_schema.sql    #cat files to reduce connects
	done

    for i  in `ls *.sql | egrep "(^create|^insert)Table_"`
	do
        cat $i >> z_schema.sql    #cat files to reduce connects
	done

    # Edit SIBME SQL files to remove CREATE STOGROUP
    echo 's/CREATE STOGROUP/-- CREATE STOGROUP/g' > z_SIBME_edit.sed
    for i  in `ls *.sql | egrep "(^create)SIBSchema"`
	do
        iconv -t IBM-1047 -f ISO8859-1 $i > o_z_SIBMEschema.ddl
        sed -f z_SIBME_edit.sed < o_z_SIBMEschema.ddl > z_SIBMEschema.ddl
        iconv -f IBM-1047 -t ISO8859-1 z_SIBMEschema.ddl > z_SIBMEschema.sql
        cat z_SIBMEschema.sql >> z_schema.sql    #cat files to reduce connects
	done

    # execute SQL Statements....
    if [ "$DBRunSQL" = "true" ]; then
       iconv -t IBM-1047 -f ISO8859-1 z_schema.sql > z_schema.ddl
       java com.ibm.db2.clp.db2 -t -f z_schema.ddl
    fi
 
    procs=$( ls *.sql | egrep "(^create)Procedure" 2> /dev/null)
    first=1
    for i in $procs;
      do
         if [[ $first -eq "1" ]]; then
             echo '\nCONNECT TO '$DBALIAS'@' > z_schemaProc.ddl
             echo $DB_OPTIONS'@\n' >> z_schemaProc.ddl
             iconv -f IBM-1047 -t ISO8859-1 z_schemaProc.ddl > z_schemaProc.sql
             rm z_schemaProc.ddl
             first=0
         fi
         cat $i >> z_schemaProc.sql    #cat files to reduce connects
       done
 
    if [[ $first -eq "0" ]]; then
      if [ "$DBRunSQL" = "true" ]; then
         iconv -t IBM-1047 -f ISO8859-1 z_schemaProc.sql > z_schemaProc.ddl
         java com.ibm.db2.clp.db2 -td@ -f z_schemaProc.ddl
      fi
    fi
fi

if [ "$DBRunSQL" = "true" ]; then
   printf "\n %d SQL statements failed, %d SQL statements executed successfully\n"  $(grep "DSNC116I" $OUTFILE | wc -l) $(grep "DSNC101I" $OUTFILE | wc -l)
fi

echo  "\nScript Complete"

cd $INIT_DIR
exit $?